<?php

namespace Drupal\cloudflare_stream\Plugin\Field\FieldFormatter;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Plugin implementation of the 'cloudflarevideo_default' formatter.
 *
 * @FieldFormatter(
 *   id = "cloudflarevideo_default",
 *   module = "cloudflare_stream",
 *   label = @Translation("Cloudflare Video"),
 *   field_types = {
 *     "cloudflarevideo"
 *   }
 * )
 */
class CloudflareVideoFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'muted' => '',
      'width' => '854',
      'height' => '480',
      'autoplay' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements['autoplay'] = [
      '#title' => $this->t('Autoplay'),
      '#type' => 'checkbox',
      '#description' => $this->t('Autoplay the Cloudflare videos'),
      '#default_value' => $this->getSetting('autoplay'),
    ];
    $elements['muted'] = [
      '#title' => $this->t('Muted Video'),
      '#type' => 'checkbox',
      '#description' => $this->t("Make the video muted."),
      '#default_value' => $this->getSetting('muted'),
    ];
    $elements['width'] = [
      '#title' => $this->t('Width'),
      '#type' => 'number',
      '#field_suffix' => 'px',
      '#default_value' => $this->getSetting('width'),
      '#required' => TRUE,
      '#size' => 20,
    ];
    $elements['height'] = [
      '#title' => $this->t('Height'),
      '#type' => 'number',
      '#field_suffix' => 'px',
      '#default_value' => $this->getSetting('height'),
      '#required' => TRUE,
      '#size' => 20,
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $dimensions = $this->getSetting('muted') ? $this->t('Muted') : $this->t('@widthx@height', ['@width' => $this->getSetting('width'), '@height' => $this->getSetting('height')]);
    $summary[] = $this->t(
      'Embedded Cloudflare Video (@dimensions@autoplay).', [
        '@dimensions' => $dimensions,
        '@autoplay' => $this->getSetting('autoplay') ? $this->t(', autoplaying') : '',
      ]
    );
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $configFactory = \Drupal::service('config.factory')->get('cloudflare_stream.settings');
    $httpClient = \Drupal::service('http_client');
    $messenger = \Drupal::service('messenger');
    foreach ($items as $delta => $item) {
      $zoneID = $configFactory->get('zone_id');
      $email = $configFactory->get('email');
      $apiKey = $configFactory->get('api_key');
      $cloudflareVideoID = $this->getVideoId($item);
      try {
        $headers = [
          'X-Auth-Email' => $email,
          'X-Auth-Key' => $apiKey,
          'Content-Type' => "application/json",
        ];

        $request = $httpClient->get("https://api.cloudflare.com/client/v4/zones/$zoneID/media/$cloudflareVideoID/embed", [
          'headers' => $headers,
        ]);

        $response = $request->getBody()->getContents();
        $elements[$delta] = [
          '#type' => 'container',
          '#theme' => 'cloudflare_video',
          '#stream' => $response,
          '#attached' => [
            'library' => [
              'cloudflare_stream/cloudflare_stream',
            ],
          ],
        ];
        $elements[$delta]['#attached']['drupalSettings'] = [
          'cloudflare_stream' => [
            'width' => $this->getSetting('width') ?? '',
            'height' => $this->getSetting('height') ?? '',
            'autoplay' => $this->getSetting('autoplay') ? 'autoplay' : '',
            'muted' => $this->getSetting('muted') ? 'muted' : '',
          ],
        ];
      }
      catch (GuzzleException $error) {
        // Get the original response.
        $response = $error->getResponse();
        // Get the info returned from the remote server.
        $response_info = $response->getBody()->getContents();
        // Using FormattableMarkup allows for the use of <pre/> tags,
        // giving a more readable log item.
        $message = new FormattableMarkup('CloudflareVideoFormatter: API connection error. Error details are as follows:<pre>@response</pre>', ['@response' => print_r(Json::decode($response_info), TRUE)]);
        // Log the error.
        watchdog_exception('Cloudflare Stream', $error, $message);
        $messenger->addError($message);
      }
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    $elements = $this->viewElements($items, $langcode);
    if (!isset($elements) || is_null($elements)) {
      return [];
    }
    return parent::view($items, $langcode);
  }

  /**
   * Returns the video id on cloudflare stream.
   */
  protected function getVideoId($item) {
    $videoID = $item->get('cloudflareStreamVideoID')->getValue();

    return $videoID;
  }

}
